@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2019 Savoir-faire Linux Inc.
  *
  *  Authors: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *           Romain Bertozzi <romain.bertozzi@savoirfairelinux.com>
@@ -22,20 +22,21 @@ package cx.ring.fragments;
 
 import android.app.Activity;
 import android.content.Intent;
-import android.graphics.drawable.Drawable;
 import android.net.Uri;
 import android.os.Build;
 import android.os.Bundle;
-import android.os.Handler;
-import android.support.annotation.Nullable;
-import android.support.design.widget.FloatingActionButton;
-import android.support.design.widget.Snackbar;
-import android.support.v7.app.AlertDialog;
-import android.support.v7.widget.LinearLayoutManager;
-import android.support.v7.widget.RecyclerView;
-import android.support.v7.widget.SearchView;
-import android.support.v7.widget.Toolbar;
+
+import androidx.annotation.NonNull;
+import androidx.annotation.Nullable;
+import com.google.android.material.floatingactionbutton.FloatingActionButton;
+import com.google.android.material.snackbar.Snackbar;
+import androidx.appcompat.app.AlertDialog;
+import androidx.recyclerview.widget.LinearLayoutManager;
+import androidx.recyclerview.widget.RecyclerView;
+import androidx.appcompat.widget.SearchView;
+import androidx.appcompat.widget.Toolbar;
 import android.text.InputType;
+import android.util.Log;
 import android.view.Menu;
 import android.view.MenuInflater;
 import android.view.MenuItem;
@@ -47,12 +48,7 @@ import android.widget.ImageView;
 import android.widget.ProgressBar;
 import android.widget.TextView;
 
-import com.bumptech.glide.Glide;
-import com.bumptech.glide.load.resource.drawable.DrawableTransitionOptions;
-import com.google.zxing.integration.android.IntentIntegrator;
-import com.google.zxing.integration.android.IntentResult;
-
-import java.util.ArrayList;
+import java.util.List;
 
 import butterknife.BindView;
 import butterknife.OnClick;
@@ -61,12 +57,12 @@ import cx.ring.adapters.SmartListAdapter;
 import cx.ring.client.CallActivity;
 import cx.ring.client.ConversationActivity;
 import cx.ring.client.HomeActivity;
-import cx.ring.client.QRCodeScannerActivity;
+import cx.ring.client.QRCodeActivity;
 import cx.ring.contacts.AvatarFactory;
 import cx.ring.dependencyinjection.RingInjectionComponent;
 import cx.ring.model.CallContact;
 import cx.ring.model.Conversation;
-import cx.ring.mvp.BaseFragment;
+import cx.ring.mvp.BaseSupportFragment;
 import cx.ring.smartlist.SmartListPresenter;
 import cx.ring.smartlist.SmartListView;
 import cx.ring.smartlist.SmartListViewModel;
@@ -75,7 +71,7 @@ import cx.ring.utils.ClipboardHelper;
 import cx.ring.utils.DeviceUtils;
 import cx.ring.viewholders.SmartListViewHolder;
 
-public class SmartListFragment extends BaseFragment<SmartListPresenter> implements SearchView.OnQueryTextListener,
+public class SmartListFragment extends BaseSupportFragment<SmartListPresenter> implements SearchView.OnQueryTextListener,
         SmartListViewHolder.SmartListListeners,
         Conversation.ConversationActionCallback,
         ClipboardHelper.ClipboardHelperCallback,
@@ -83,6 +79,7 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
 
     public static final String TAG = SmartListFragment.class.getSimpleName();
     private static final String STATE_LOADING = TAG + ".STATE_LOADING";
+    public static final String KEY_ACCOUNT_ID = "accountId";
 
     @BindView(R.id.newconv_fab)
     protected FloatingActionButton mFloatingActionButton;
@@ -116,9 +113,9 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
 
     private Boolean isTabletMode = false;
 
+    @Override
     public void onResume() {
         super.onResume();
-
         ((HomeActivity) getActivity()).setToolbarState(false, R.string.app_name);
 
         presenter.refresh();
@@ -135,7 +132,7 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
             @Override
             public boolean onMenuItemActionCollapse(MenuItem item) {
                 mDialpadMenuItem.setVisible(false);
-                displayFloatingActionButtonWithDelay(true, 50);
+                mFloatingActionButton.show();
                 setOverflowMenuVisible(menu, true);
                 return true;
             }
@@ -143,7 +140,7 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
             @Override
             public boolean onMenuItemActionExpand(MenuItem item) {
                 mDialpadMenuItem.setVisible(true);
-                displayFloatingActionButtonWithDelay(false, 0);
+                mFloatingActionButton.hide();
                 setOverflowMenuVisible(menu, false);
                 return true;
             }
@@ -234,7 +231,7 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
     }
 
     @Override
-    public void onViewCreated(View view, Bundle savedInstanceState) {
+    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
         setHasOptionsMenu(true);
         super.onViewCreated(view, savedInstanceState);
 
@@ -260,26 +257,12 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
         presenter.fabButtonClicked();
     }
 
-    @Override
-    public void onActivityResult(int requestCode, int resultCode, Intent data) {
-        super.onActivityResult(requestCode, resultCode, data);
-        if (requestCode == IntentIntegrator.REQUEST_CODE) {
-            IntentResult scanResult = IntentIntegrator.parseActivityResult(requestCode, resultCode, data);
-            if (scanResult != null && resultCode == Activity.RESULT_OK) {
-                String contact_uri = scanResult.getContents();
-                presenter.startConversation(CallContact.buildUnknown(contact_uri));
-            }
-        }
-    }
-
     @Override
     public void setLoading(final boolean loading) {
-        getActivity().runOnUiThread(() -> {
-            if (mLoader == null) {
-                return;
-            }
-            mLoader.setVisibility(loading ? View.VISIBLE : View.GONE);
-        });
+        if (mLoader == null) {
+            return;
+        }
+        mLoader.setVisibility(loading ? View.VISIBLE : View.GONE);
     }
 
     /**
@@ -298,22 +281,14 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
         }
     }
 
-    /**
-     * Hides or displays the floating action button after a delay
-     *
-     * @param visible true to display, false to hide
-     * @param delay   time in ms
-     */
-    private void displayFloatingActionButtonWithDelay(boolean visible, int delay) {
-        if (this.mFloatingActionButton != null) {
-            final int visibility = (visible) ? View.VISIBLE : View.GONE;
-            new Handler().postDelayed(() -> mFloatingActionButton.setVisibility(visibility), delay);
-        }
+    @Override
+    public void removeConversation(CallContact callContact) {
+        presenter.removeConversation(callContact);
     }
 
     @Override
-    public void deleteConversation(CallContact callContact) {
-        presenter.deleteConversation(callContact);
+    public void clearConversation(CallContact callContact) {
+        presenter.clearConversation(callContact);
     }
 
     @Override
@@ -350,49 +325,36 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
 
     @Override
     public void displayNetworkErrorPanel() {
-        getActivity().runOnUiThread(() -> showErrorPanel(R.string.error_no_network, false, 0, null));
+        showErrorPanel(R.string.error_no_network, false, 0, null);
     }
 
     @Override
     public void displayMobileDataPanel() {
-        getActivity().runOnUiThread(() -> showErrorPanel(R.string.error_mobile_network_available_but_disabled,
+        showErrorPanel(R.string.error_mobile_network_available_but_disabled,
                 true,
                 R.drawable.ic_settings_white,
                 v -> {
                     Activity activity = getActivity();
-                    if (activity != null && activity instanceof HomeActivity) {
+                    if (activity instanceof HomeActivity) {
                         HomeActivity homeActivity = (HomeActivity) activity;
                         homeActivity.goToSettings();
                     }
-                }));
+                });
     }
 
     @Override
     public void displayContact(final CallContact contact) {
-        getActivity().runOnUiThread(() -> {
-            if (mNewContact == null) {
-                return;
-            }
+        if (mNewContact == null) {
+            return;
+        }
 
-            TextView display_name = mNewContact.findViewById(R.id.display_name);
-            display_name.setText(contact.getRingUsername());
+        TextView display_name = mNewContact.findViewById(R.id.display_name);
+        display_name.setText(contact.getRingUsername());
 
-            ImageView photo = mNewContact.findViewById(R.id.photo);
+        ImageView photo = mNewContact.findViewById(R.id.photo);
 
-            Drawable contactPicture = AvatarFactory.getAvatar(
-                    getActivity(),
-                    contact.getPhoto(),
-                    contact.getRingUsername(),
-                    contact.getIds().get(0));
-
-            Glide.with(getActivity())
-                    .load(contactPicture)
-                    .apply(AvatarFactory.getGlideOptions(true, false))
-                    .transition(DrawableTransitionOptions.withCrossFade())
-                    .into(photo);
-
-            mNewContact.setVisibility(View.VISIBLE);
-        });
+        AvatarFactory.loadGlideAvatar(photo, contact);
+        mNewContact.setVisibility(View.VISIBLE);
     }
 
     @Override
@@ -424,17 +386,25 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
                         case ActionHelper.ACTION_COPY:
                             presenter.copyNumber(smartListViewModel);
                             break;
+                        case ActionHelper.ACTION_CLEAR:
+                            presenter.clearConversation(smartListViewModel);
+                            break;
                         case ActionHelper.ACTION_DELETE:
-                            presenter.deleteConversation(smartListViewModel);
+                            presenter.removeConversation(smartListViewModel);
                             break;
                         case ActionHelper.ACTION_BLOCK:
-                            presenter.removeContact(smartListViewModel);
+                            presenter.banContact(smartListViewModel);
                             break;
                     }
                 })
                 .show();
     }
 
+    @Override
+    public void displayClearDialog(CallContact callContact) {
+        ActionHelper.launchClearAction(getActivity(), callContact, SmartListFragment.this);
+    }
+
     @Override
     public void displayDeleteDialog(CallContact callContact) {
         ActionHelper.launchDeleteAction(getActivity(), callContact, SmartListFragment.this);
@@ -454,22 +424,18 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
 
     @Override
     public void hideSearchRow() {
-        getActivity().runOnUiThread(() -> {
-            if (mNewContact == null) {
-                return;
-            }
-            mNewContact.setVisibility(View.GONE);
-        });
+        if (mNewContact == null) {
+            return;
+        }
+        mNewContact.setVisibility(View.GONE);
     }
 
     @Override
     public void hideErrorPanel() {
-        getActivity().runOnUiThread(() -> {
-            if (mErrorMessagePane == null) {
-                return;
-            }
-            mErrorMessagePane.setVisibility(View.GONE);
-        });
+        if (mErrorMessagePane == null) {
+            return;
+        }
+        mErrorMessagePane.setVisibility(View.GONE);
     }
 
     @Override
@@ -483,21 +449,38 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
     }
 
     @Override
-    public void updateList(final ArrayList<SmartListViewModel> smartListViewModels) {
+    public void updateList(@Nullable final List<SmartListViewModel> smartListViewModels) {
+        if (mRecyclerView == null)
+            return;
         if (mRecyclerView.getAdapter() == null) {
             mSmartListAdapter = new SmartListAdapter(smartListViewModels, SmartListFragment.this);
             mRecyclerView.setAdapter(mSmartListAdapter);
             mRecyclerView.setHasFixedSize(true);
             LinearLayoutManager llm = new LinearLayoutManager(getActivity());
-            llm.setOrientation(LinearLayoutManager.VERTICAL);
+            llm.setOrientation(RecyclerView.VERTICAL);
             mRecyclerView.setLayoutManager(llm);
+        } else {
+            mSmartListAdapter.update(smartListViewModels);
         }
         mRecyclerView.setVisibility(View.VISIBLE);
-        mSmartListAdapter.update(smartListViewModels);
     }
 
     @Override
-    public void goToConversation(String accountId, String contactId) {
+    public void update(int position) {
+        Log.w(TAG, "update " + position + " " + mSmartListAdapter);
+        if (mSmartListAdapter != null) {
+            mSmartListAdapter.notifyItemChanged(position);
+        }
+    }
+
+    @Override
+    public void update(SmartListViewModel model) {
+        if (mSmartListAdapter != null)
+            mSmartListAdapter.update(model);
+    }
+
+    @Override
+    public void goToConversation(String accountId, cx.ring.model.Uri contactId) {
         if (mSearchMenuItem != null) {
             mSearchMenuItem.collapseActionView();
         }
@@ -507,11 +490,11 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
                     .setClass(getActivity(), ConversationActivity.class)
                     .setAction(Intent.ACTION_VIEW)
                     .putExtra(ConversationFragment.KEY_ACCOUNT_ID, accountId)
-                    .putExtra(ConversationFragment.KEY_CONTACT_RING_ID, contactId);
+                    .putExtra(ConversationFragment.KEY_CONTACT_RING_ID, contactId.toString());
             startActivity(intent);
         } else {
             Bundle bundle = new Bundle();
-            bundle.putString(ConversationFragment.KEY_CONTACT_RING_ID, contactId);
+            bundle.putString(ConversationFragment.KEY_CONTACT_RING_ID, contactId.toString());
             bundle.putString(ConversationFragment.KEY_ACCOUNT_ID, accountId);
             ((HomeActivity) getActivity()).startConversationTablet(bundle);
         }
@@ -529,7 +512,13 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
 
     @Override
     public void goToQRActivity() {
-        QRCodeScannerActivity.startQRCodeScanWithFragmentReceiver(this);
+        Intent i = new Intent(getActivity(), QRCodeActivity.class);
+        if(presenter != null) {
+            Bundle bundle = new Bundle();
+            bundle.putString(KEY_ACCOUNT_ID, presenter.getAccountID());
+            i.putExtras(bundle);
+        }
+        startActivity(i);
     }
 
     @Override
@@ -539,12 +528,8 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
 
     @Override
     public void scrollToTop() {
-        getActivity().runOnUiThread(() -> {
-            if (mRecyclerView == null) {
-                return;
-            }
+        if (mRecyclerView != null)
             mRecyclerView.scrollToPosition(0);
-        });
     }
 
     @Override
@@ -556,9 +541,4 @@ public class SmartListFragment extends BaseFragment<SmartListPresenter> implemen
     public void onItemLongClick(SmartListViewModel smartListViewModel) {
         presenter.conversationLongClicked(smartListViewModel);
     }
-
-    @Override
-    public void onPhotoClick(SmartListViewModel smartListViewModel) {
-        presenter.photoClicked(smartListViewModel);
-    }
 }
