@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2019 Savoir-faire Linux Inc.
  *
  *  Author: Hadrien De Sousa <hadrien.desousa@savoirfairelinux.com>
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
@@ -33,8 +33,8 @@ import android.os.Build;
 
 import androidx.annotation.Nullable;
 
-import android.view.Surface;
 import android.view.SurfaceHolder;
+import android.view.TextureView;
 import android.view.WindowManager;
 
 import java.io.File;
@@ -45,6 +45,7 @@ import java.util.Map;
 
 import androidx.media.AudioAttributesCompat;
 import cx.ring.daemon.IntVect;
+import cx.ring.daemon.Ringservice;
 import cx.ring.daemon.StringMap;
 import cx.ring.daemon.UintVect;
 import cx.ring.utils.BluetoothWrapper;
@@ -60,16 +61,14 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
     private static final Point VIDEO_SIZE_DEFAULT = new Point(720, 480);
     private static final Point VIDEO_SIZE_HD = new Point(1280, 720);
     private static final Point VIDEO_SIZE_FULL_HD = new Point(1920, 1080);
+    private static final Point VIDEO_SIZE_ULTRA_HD = new Point(3840, 2160);
 
-    public static final int VIDEO_WIDTH = 640;
-    public static final int VIDEO_HEIGHT = 480;
-
-    private static final String TAG = HardwareServiceImpl.class.getName();
-    private static WeakReference<Object> mCameraPreviewSurface = new WeakReference<>(null);
-    private static final Map<String, WeakReference<SurfaceHolder>> videoSurfaces = Collections.synchronizedMap(new HashMap<String, WeakReference<SurfaceHolder>>());
+    private static final String TAG = HardwareServiceImpl.class.getSimpleName();
+    private static WeakReference<TextureView> mCameraPreviewSurface = new WeakReference<>(null);
+    private static final Map<String, WeakReference<SurfaceHolder>> videoSurfaces = Collections.synchronizedMap(new HashMap<>());
     private final Map<String, Shm> videoInputs = new HashMap<>();
     private final Context mContext;
-    private final CameraService cameraService;
+    private final CameraServiceCamera2 cameraService;
     private final Ringer mRinger;
     private final AudioManager mAudioManager;
     private BluetoothWrapper mBluetoothWrapper;
@@ -85,16 +84,12 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
         mAudioManager = (AudioManager) mContext.getSystemService(Context.AUDIO_SERVICE);
         mHasSpeakerPhone = hasSpeakerphone();
         mRinger = new Ringer(mContext);
-        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
-            cameraService = new CameraServiceCamera2(mContext);
-        } else {
-            cameraService = new CameraServiceKitKat();
-        }
+        cameraService = new CameraServiceCamera2(mContext);
     }
 
     public void initVideo() {
         Log.i(TAG, "initVideo()");
-        cameraService.init();
+        mExecutor.submit(cameraService::init);
     }
 
     public boolean isVideoAvailable() {
@@ -381,7 +376,7 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
             Log.w(TAG, "startCapture: no video parameters ");
             return;
         }
-        final Object surface = mCameraPreviewSurface.get();
+        final TextureView surface = mCameraPreviewSurface.get();
         if (surface == null) {
             Log.w(TAG, "Can't start capture: no surface registered.");
             cameraService.setPreviewParams(videoParams);
@@ -405,14 +400,14 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
                 public void onError() {
                     stopCapture();
                 }
-            });
+            }, mPreferenceService.getUserSettings().isHwEncoding());
         });
         cameraService.setPreviewParams(videoParams);
         VideoEvent event = new VideoEvent();
         event.started = true;
-        boolean s = videoParams.rotation % 180 != 0;
-        event.w = s ? videoParams.height : videoParams.width;
-        event.h = s ? videoParams.width : videoParams.height;
+        event.w = videoParams.width;
+        event.h = videoParams.height;
+        event.rot = videoParams.rotation;
         videoEvents.onNext(event);
     }
 
@@ -423,6 +418,10 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
         endCapture();
     }
 
+    public void requestKeyFrame() {
+        cameraService.requestKeyFrame();
+    }
+
     public void endCapture() {
         if (cameraService.isOpen()) {
             //final CameraService.VideoParams params = previewParams;
@@ -471,15 +470,14 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
 
     @Override
     public void addPreviewVideoSurface(Object oholder) {
-        /*if (!(oholder instanceof SurfaceHolder)) {
+        if (!(oholder instanceof TextureView)) {
             return;
         }
-        SurfaceHolder holder = (SurfaceHolder)oholder;*/
-        //Surface holder = (Surface) oholder;
-        Log.w(TAG, "addPreviewVideoSurface " + oholder.hashCode() + " mCapturingId " + mCapturingId);
+        TextureView holder = (TextureView)oholder;
+        Log.w(TAG, "addPreviewVideoSurface " + holder.hashCode() + " mCapturingId " + mCapturingId);
         if (mCameraPreviewSurface.get() == oholder)
             return;
-        mCameraPreviewSurface = new WeakReference<>(oholder);
+        mCameraPreviewSurface = new WeakReference<>(holder);
         if (mShouldCapture && !mIsCapturing) {
             startCapture(mCapturingId);
         }
@@ -515,12 +513,17 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
     }
 
     @Override
-    public void switchInput(String id) {
+    public void switchInput(String id, boolean setDefaultCamera) {
         Log.w(TAG, "switchInput " + id);
-        String camId = cameraService.switchInput();
-        final StringMap map = cameraService.getNativeParams(camId).toMap(mContext.getResources().getConfiguration().orientation);
-        final String uri = "camera://" + camId;
-        switchInput(id, uri, map);
+        String camId = cameraService.switchInput(setDefaultCamera);
+        try {
+            final StringMap map = cameraService.getNativeParams(camId).toMap();
+            final String uri = "camera://" + camId;
+            switchInput(id, uri, map);
+        }
+        catch(Exception e) {
+            Log.e(TAG, "Error with hardware service, switchInput: " + e.getMessage());
+        }
     }
 
     @Override
@@ -530,14 +533,13 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
         setPreviewSettings();
         String currentCamera = cameraService.getCurrentCamera();
         final String uri = "camera://" + currentCamera;
-        final StringMap map = cameraService.getNativeParams(currentCamera).toMap(mContext.getResources().getConfiguration().orientation);
+        final StringMap map = cameraService.getNativeParams(currentCamera).toMap();
         switchInput(id, uri, map);
     }
 
     @Override
     public void setPreviewSettings() {
-        int orientation = mContext.getResources().getConfiguration().orientation;
-        setPreviewSettings(cameraService.getPreviewSettings(orientation));
+        setPreviewSettings(cameraService.getPreviewSettings());
     }
 
     @Override
@@ -550,6 +552,25 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
         return cameraService.isPreviewFromFrontCamera();
     }
 
+    @Override
+    public void setDeviceOrientation(int rotation) {
+        cameraService.setOrientation(rotation);
+        if (mCapturingId != null) {
+            CameraService.VideoParams videoParams = cameraService.getParams(mCapturingId);
+            VideoEvent event = new VideoEvent();
+            event.started = true;
+            event.w = videoParams.width;
+            event.h = videoParams.height;
+            event.rot = videoParams.rotation;
+            videoEvents.onNext(event);
+        }
+    }
+
+    @Override
+    protected String[] getVideoDevices() {
+        return cameraService.getCameraIds();
+    }
+
     private static class Shm {
         String id;
         int w, h;
