@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2019 Savoir-faire Linux Inc.
  *
  *  Author: Hadrien De Sousa <hadrien.desousa@savoirfairelinux.com>
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
@@ -20,47 +20,46 @@
  */
 package cx.ring.call;
 
-import java.util.concurrent.Executors;
-import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
 
 import javax.inject.Inject;
 import javax.inject.Named;
 
-import cx.ring.model.CallContact;
-import cx.ring.model.Conference;
 import cx.ring.model.SipCall;
 import cx.ring.mvp.RootPresenter;
 import cx.ring.services.AccountService;
 import cx.ring.services.CallService;
 import cx.ring.services.ContactService;
+import cx.ring.services.DeviceRuntimeService;
 import cx.ring.services.HardwareService;
 import cx.ring.services.NotificationService;
 import cx.ring.utils.Log;
 import cx.ring.utils.StringUtils;
 import io.reactivex.Scheduler;
+import io.reactivex.disposables.Disposable;
 
 public class CallPresenter extends RootPresenter<CallView> {
 
     public final static String TAG = CallPresenter.class.getSimpleName();
 
     private AccountService mAccountService;
+    private ContactService mContactService;
     private NotificationService mNotificationService;
     private HardwareService mHardwareService;
     private CallService mCallService;
-    private ContactService mContactService;
+    private DeviceRuntimeService mDeviceRuntimeService;
 
     private SipCall mSipCall;
     private boolean mOnGoingCall = false;
     private boolean mAudioOnly = true;
+    private boolean permissionChanged = false;
 
     private int videoWidth = -1;
     private int videoHeight = -1;
     private int previewWidth = -1;
     private int previewHeight = -1;
 
-    private ScheduledExecutorService executor;
-    private Runnable timeRunnable = this::updateTime;
+    private Disposable timeUpdateTask = null;
 
     @Inject
     @Named("UiScheduler")
@@ -68,23 +67,38 @@ public class CallPresenter extends RootPresenter<CallView> {
 
     @Inject
     public CallPresenter(AccountService accountService,
+                         ContactService contactService,
                          NotificationService notificationService,
                          HardwareService hardwareService,
-                         CallService callService,
-                         ContactService contactService) {
+                         CallService callService, DeviceRuntimeService deviceRuntimeService) {
         mAccountService = accountService;
+        mContactService = contactService;
         mNotificationService = notificationService;
         mHardwareService = hardwareService;
         mCallService = callService;
-        mContactService = contactService;
+        mDeviceRuntimeService = deviceRuntimeService;
     }
 
+    public void cameraPermissionChanged(boolean isGranted) {
+        if (isGranted && mHardwareService.isVideoAvailable()) {
+            mHardwareService.initVideo();
+            permissionChanged = true;
+        }
+    }
+
+    public void audioPermissionChanged(boolean isGranted) {
+        if(isGranted && mHardwareService.hasMicrophone()) {
+            mCallService.setAudioPlugin(mCallService.getCurrentAudioOutputPlugin());
+        }
+    }
+
+
     @Override
     public void unbindView() {
-        super.unbindView();
         if (!mAudioOnly) {
-            mHardwareService.stopCapture();
+            mHardwareService.endCapture();
         }
+        super.unbindView();
     }
 
     @Override
@@ -100,6 +114,13 @@ public class CallPresenter extends RootPresenter<CallView> {
         mCompositeDisposable.add(mHardwareService.getVideoEvents()
                 .observeOn(mUiScheduler)
                 .subscribe(this::onVideoEvent));
+        mCompositeDisposable.add(mHardwareService
+                .getBluetoothEvents()
+                .subscribe(event -> {
+                    if (!event.connected && mSipCall != null) {
+                        hangupCall();
+                    }
+                }));
     }
 
     public void initOutGoing(String accountId, String contactRingId, boolean audioOnly) {
@@ -111,31 +132,25 @@ public class CallPresenter extends RootPresenter<CallView> {
         if (mHardwareService.getCameraCount() == 0) {
             audioOnly = true;
         }
-        getView().blockScreenRotation();
+        //getView().blockScreenRotation();
 
         mCompositeDisposable.add(mCallService
                 .placeCallObservable(accountId, StringUtils.toNumber(contactRingId), audioOnly)
                 .observeOn(mUiScheduler)
-                .subscribe(call ->  {
-            mSipCall = call;
-            mAudioOnly = mSipCall.isAudioOnly();
-            getView().updateMenu();
-            confUpdate();
-            getContactDetails();
-        }));
+                .subscribe(call -> {
+                    contactUpdate(call);
+                    confUpdate(call);
+                }, e -> finish()));
     }
 
     public void initIncoming(String confId) {
-        getView().blockScreenRotation();
+        //getView().blockScreenRotation();
         mCompositeDisposable.add(mCallService.getCallUpdates(confId)
                 .observeOn(mUiScheduler)
                 .subscribe(call -> {
-            mSipCall = call;
-            mAudioOnly = mSipCall.isAudioOnly();
-            getView().updateMenu();
-            confUpdate();
-            getContactDetails();
-        }));
+                    contactUpdate(call);
+                    confUpdate(call);
+                }, e -> finish()));
     }
 
     public void prepareOptionMenu() {
@@ -156,23 +171,26 @@ public class CallPresenter extends RootPresenter<CallView> {
         getView().goToConversation(mSipCall.getAccount(), mSipCall.getContact().getIds().get(0));
     }
 
-    public void speakerClick() {
-        mHardwareService.toggleSpeakerphone();
+    public void speakerClick(boolean checked) {
+        mHardwareService.toggleSpeakerphone(checked);
+    }
+
+    public void muteMicrophoneToggled(boolean checked) {
+        mCallService.setMuted(checked);
+    }
+
+
+    public boolean isMicrophoneMuted() {
+        return mCallService.isCaptureMuted();
     }
 
     public void switchVideoInputClick() {
-        mHardwareService.switchInput(mSipCall.getCallId());
+        mHardwareService.switchInput(mSipCall.getCallId(), false);
         getView().switchCameraIcon(mHardwareService.isPreviewFromFrontCamera());
     }
 
-    public void screenRotationClick() {
-        getView().changeScreenRotation();
-    }
-
-    public void configurationChanged() {
-        if (mSipCall != null) {
-            mHardwareService.restartCamera(mSipCall.getCallId());
-        }
+    public void configurationChanged(int rotation) {
+        mHardwareService.setDeviceOrientation(rotation);
     }
 
     public void dialpadClick() {
@@ -212,7 +230,7 @@ public class CallPresenter extends RootPresenter<CallView> {
 
     public void previewVideoSurfaceCreated(Object holder) {
         mHardwareService.addPreviewVideoSurface(holder);
-        mHardwareService.startCapture(null);
+        //mHardwareService.startCapture(null);
     }
 
     public void videoSurfaceDestroyed() {
@@ -224,61 +242,76 @@ public class CallPresenter extends RootPresenter<CallView> {
 
     public void previewVideoSurfaceDestroyed() {
         mHardwareService.removePreviewVideoSurface();
-        mHardwareService.stopCapture();
+        mHardwareService.endCapture();
     }
 
     public void displayChanged() {
-        mHardwareService.switchInput(mSipCall.getCallId());
+        mHardwareService.switchInput(mSipCall.getCallId(), false);
     }
 
     public void layoutChanged() {
-        getView().resetVideoSize(videoWidth, videoHeight, previewWidth, previewHeight);
+        //getView().resetVideoSize(videoWidth, videoHeight, previewWidth, previewHeight);
     }
 
+
     public void uiVisibilityChanged(boolean displayed) {
-        getView().displayHangupButton(mOnGoingCall && displayed);
+        CallView view = getView();
+        if (view != null)
+            view.displayHangupButton(mOnGoingCall && displayed);
     }
 
     private void finish() {
-        if (executor != null && !executor.isShutdown()) {
-            executor.shutdown();
+        if (timeUpdateTask != null && !timeUpdateTask.isDisposed()) {
+            timeUpdateTask.dispose();
+            timeUpdateTask = null;
         }
         mSipCall = null;
-        getView().finish();
+        CallView view = getView();
+        if (view != null)
+            view.finish();
     }
 
-    private void confUpdate() {
-        if (mSipCall == null) {
-            return;
+    private void contactUpdate(final SipCall call) {
+        if (mSipCall != call) {
+            mSipCall = call;
+            mCompositeDisposable.add(mContactService.observeContact(call.getAccount(), call.getContact())
+                    .observeOn(mUiScheduler)
+                    .subscribe(c -> getView().updateContactBubble(c)));
         }
+    }
+
+    private void confUpdate(SipCall call) {
+        mAudioOnly = call.isAudioOnly();
         CallView view = getView();
-        mAudioOnly = mSipCall.isAudioOnly();
-        if (mSipCall.isOnGoing()) {
+        if (view == null)
+            return;
+        view.updateMenu();
+        if (call.isOnGoing()) {
             mOnGoingCall = true;
-            view.initNormalStateDisplay(mAudioOnly);
-            view.updateContactBubble(mSipCall.getContact());
+            view.initNormalStateDisplay(mAudioOnly, mHardwareService.isSpeakerPhoneOn(), isMicrophoneMuted());
             view.updateMenu();
             if (!mAudioOnly) {
                 mHardwareService.setPreviewSettings();
                 view.displayVideoSurface(true);
+                if(permissionChanged) {
+                    mHardwareService.switchInput(mSipCall.getCallId(), permissionChanged);
+                    permissionChanged = false;
+                }
             }
-            if (executor == null || executor.isShutdown()) {
-                executor = Executors.newSingleThreadScheduledExecutor();
-                executor.scheduleAtFixedRate(timeRunnable, 0, 1, TimeUnit.SECONDS);
-            }
-        } else if (mSipCall.isRinging()) {
-            if (mSipCall.isIncoming()) {
-                if (mAccountService.getAccount(mSipCall.getAccount()).isAutoanswerEnabled()) {
-                    mCallService.accept(mSipCall.getCallId());
+            if (timeUpdateTask != null)
+                timeUpdateTask.dispose();
+            timeUpdateTask = mUiScheduler.schedulePeriodicallyDirect(this::updateTime, 0, 1, TimeUnit.SECONDS);
+        } else if (call.isRinging()) {
+            if (call.isIncoming()) {
+                if (mAccountService.getAccount(call.getAccount()).isAutoanswerEnabled()) {
+                    mCallService.accept(call.getCallId());
                 } else {
                     view.initIncomingCallDisplay();
-                    view.updateContactBubble(mSipCall.getContact());
                 }
             } else {
                 mOnGoingCall = false;
-                view.updateCallStatus(mSipCall.getCallState());
+                view.updateCallStatus(call.getCallState());
                 view.initOutGoingCallDisplay();
-                view.updateContactBubble(mSipCall.getContact());
             }
         } else {
             finish();
@@ -286,49 +319,18 @@ public class CallPresenter extends RootPresenter<CallView> {
     }
 
     private void updateTime() {
-        if (mSipCall != null) {
+        CallView view = getView();
+        if (view != null && mSipCall != null) {
             long duration = System.currentTimeMillis() - mSipCall.getTimestampStart();
             duration = duration / 1000;
             if (mSipCall.isOnGoing()) {
-                getView().updateTime(duration);
+                view.updateTime(duration);
             }
         }
     }
 
-    private void getContactDetails() {
-        if (mSipCall == null) {
-            Log.e(TAG, "Not able to get SIP call");
-            return;
-        }
-        CallContact callContact = mSipCall.getContact();
-        if (mContactService == null) {
-            Log.e(TAG, "Not able to get contact service");
-            return;
-        }
-        mContactService.loadContactData(callContact);
-
-        getView().updateContactBubble(callContact);
-    }
-
-    private void parseCall(SipCall call, int callState) {
-        if (mSipCall == null || mSipCall != call) {
-            return;
-        }
-
-        if (callState == SipCall.State.HUNGUP
-                || callState == SipCall.State.BUSY
-                || callState == SipCall.State.FAILURE
-                || callState == SipCall.State.OVER) {
-            finish();
-        } else if (callState != SipCall.State.INACTIVE) {
-            mNotificationService.showCallNotification(new Conference(mSipCall));
-        }
-    }
-
     private void onVideoEvent(HardwareService.VideoEvent event) {
-        Log.d(TAG, "VIDEO_EVENT: " + event.start + " " + event.callId);
-        previewHeight = event.w;
-        previewWidth = event.h;
+        Log.d(TAG, "VIDEO_EVENT: " + event.start + " " + event.callId + " " + event.w + "x" + event.h);
 
         if (event.start) {
             getView().displayVideoSurface(true);
@@ -337,9 +339,18 @@ public class CallPresenter extends RootPresenter<CallView> {
             if (event.started) {
                 videoWidth = event.w;
                 videoHeight = event.h;
+                getView().resetVideoSize(videoWidth, videoHeight);
+            }
+        } else if (event.callId == null) {
+            if (event.started) {
+                previewWidth = event.w;
+                previewHeight = event.h;
+                getView().resetPreviewVideoSize(previewWidth, previewHeight, event.rot);
             }
         }
-        getView().resetVideoSize(videoWidth, videoHeight, previewWidth, previewHeight);
+        /*if (event.started || event.start) {
+            getView().resetVideoSize(videoWidth, videoHeight, previewWidth, previewHeight);
+        }*/
     }
 
     public void positiveButtonClicked() {
@@ -364,8 +375,12 @@ public class CallPresenter extends RootPresenter<CallView> {
         }
     }
 
+    public boolean isAudioOnly() {
+        return mAudioOnly;
+    }
+
     public void requestPipMode() {
-        if (mSipCall.isOnGoing() && !mSipCall.isAudioOnly()) {
+        if (mSipCall != null && mSipCall.isOnGoing() && !mSipCall.isAudioOnly()) {
             getView().enterPipMode(mSipCall);
         }
     }
@@ -378,4 +393,12 @@ public class CallPresenter extends RootPresenter<CallView> {
             getView().displayPreviewSurface(true);
         }
     }
+
+    public boolean isSpeakerphoneOn() {
+        return mHardwareService.isSpeakerPhoneOn();
+    }
+
+    public void sendDtmf(CharSequence s) {
+        mCallService.playDtmf(s.toString());
+    }
 }
