@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2019 Savoir-faire Linux Inc.
  *
  *  Author: Hadrien De Sousa <hadrien.desousa@savoirfairelinux.com>
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
@@ -58,8 +58,7 @@ import io.reactivex.subjects.Subject;
 public class ConversationPresenter extends RootPresenter<ConversationView> {
 
     private static final String TAG = ConversationPresenter.class.getSimpleName();
-    private ContactService mContactService;
-
+    private final ContactService mContactService;
     private final AccountService mAccountService;
     private final HardwareService mHardwareService;
     private final ConversationFacade mConversationFacade;
@@ -102,15 +101,15 @@ public class ConversationPresenter extends RootPresenter<ConversationView> {
             initView(mConversation, view);
     }
 
-    public void init(String contactRingId, String accountId) {
-        init(new Uri(contactRingId), accountId);
-    }
-
     public void init(Uri contactRingId, String accountId) {
         Log.w(TAG, "init " + contactRingId + " " + accountId);
         mContactRingId = contactRingId;
         mAccountId = accountId;
-        mCompositeDisposable.add(mConversationFacade.startConversation(accountId, contactRingId)
+        Account account = mAccountService.getAccount(accountId);
+        if (account != null)
+            initContact(account, contactRingId, getView());
+        mCompositeDisposable.add(mConversationFacade
+                .startConversation(accountId, contactRingId)
                 .observeOn(mUiScheduler)
                 .subscribe(this::setConversation, e -> getView().goToHome()));
     }
@@ -135,10 +134,12 @@ public class ConversationPresenter extends RootPresenter<ConversationView> {
     public void resume() {
         Log.w(TAG, "resume " + mConversation + " " + mAccountId + " " + mContactRingId);
         mVisibilityDisposable.clear();
-        mVisibilityDisposable.add(mConversationSubject.firstOrError().subscribe(conversation -> {
-            conversation.setVisible(true);
-            mConversationFacade.readMessages(mAccountService.getAccount(mAccountId), conversation);
-        }));
+        mVisibilityDisposable.add(mConversationSubject
+                .firstOrError()
+                .subscribe(conversation -> {
+                    conversation.setVisible(true);
+                    mConversationFacade.readMessages(mAccountService.getAccount(mAccountId), conversation);
+                }));
     }
 
     private CallContact initContact(final Account account, final Uri uri, final ConversationView view) {
@@ -150,12 +151,12 @@ public class ConversationPresenter extends RootPresenter<ConversationView> {
                 contact = account.getContactFromCache(uri);
                 TrustRequest req = account.getRequest(rawId);
                 if (req == null) {
-                    getView().switchToUnknownView(contact.getRingUsername());
+                    view.switchToUnknownView(contact.getRingUsername());
                 } else {
-                    getView().switchToIncomingTrustRequestView(req.getDisplayname());
+                    view.switchToIncomingTrustRequestView(req.getDisplayname());
                 }
             } else {
-                getView().switchToConversationView();
+                view.switchToConversationView();
             }
             Log.w(TAG, "initContact " + contact.getUsername());
             if (contact.getUsername() == null) {
@@ -163,10 +164,9 @@ public class ConversationPresenter extends RootPresenter<ConversationView> {
             }
         } else {
             contact = mContactService.findContact(account, uri);
-            getView().switchToConversationView();
+            view.switchToConversationView();
         }
-        view.displayContactPhoto(contact.getPhoto());
-        view.displayContactName(contact);
+        view.displayContact(contact);
         return contact;
     }
 
@@ -180,41 +180,34 @@ public class ConversationPresenter extends RootPresenter<ConversationView> {
         view.hideNumberSpinner();
 
         Account account = mAccountService.getAccount(mAccountId);
-        CallContact contact = initContact(account, mContactRingId, view);
 
         mConversationDisposable.add(c.getSortedHistory()
-                .subscribe(view::refreshView));
-        mConversationDisposable.add(contact.getUpdates()
+                .subscribe(view::refreshView, e -> Log.e(TAG, "Can't update element", e)));
+        mConversationDisposable.add(c.getCleared()
                 .observeOn(mUiScheduler)
-                .subscribe(r -> initContact(account, mContactRingId, view)));
+                .subscribe(view::refreshView, e -> Log.e(TAG, "Can't update element", e)));
+        mConversationDisposable.add(mContactService.getLoadedContact(c.getAccountId(), c.getContact())
+                .observeOn(mUiScheduler)
+                .subscribe(contact -> initContact(account, mContactRingId, view), e -> Log.e(TAG, "Can't update element", e)));
         mConversationDisposable.add(c.getNewElements()
                 .observeOn(mUiScheduler)
-                .subscribe(view::addElement));
+                .subscribe(view::addElement, e -> Log.e(TAG, "Can't update element", e)));
         mConversationDisposable.add(c.getUpdatedElements()
                 .observeOn(mUiScheduler)
-                .subscribe(view::updateElement));
+                .subscribe(view::updateElement, e -> Log.e(TAG, "Can't update element", e)));
         mConversationDisposable.add(c.getRemovedElements()
                 .observeOn(mUiScheduler)
-                .subscribe(view::removeElement));
+                .subscribe(view::removeElement, e -> Log.e(TAG, "Can't update element", e)));
         mConversationDisposable.add(c.getCalls()
                 .observeOn(mUiScheduler)
-                .subscribe(calls -> updateOngoingCallView()));
+                .subscribe(calls -> updateOngoingCallView(), e -> Log.e(TAG, "Can't update element", e)));
+        mConversationDisposable.add(c.getColor()
+                .observeOn(mUiScheduler)
+                .subscribe(view::setConversationColor, e -> Log.e(TAG, "Can't update element", e)));
     }
 
-    public void prepareMenu() {
-        getView().displayAddContact(mConversation != null && mConversation.getContact().getId() < 0);
-    }
-
-    public void addContact() {
-        getView().goToAddContact(mConversation.getContact());
-    }
-
-    public void deleteAction() {
-        getView().displayDeleteDialog(mConversation);
-    }
-
-    public void copyToClipboard() {
-        getView().displayCopyToClipboard(mConversation.getContact());
+    public void openContact() {
+        getView().goToContactActivity(mAccountId, mConversation.getContact().getPrimaryNumber());
     }
 
     public void sendTextMessage(String message) {
@@ -223,7 +216,7 @@ public class ConversationPresenter extends RootPresenter<ConversationView> {
         }
         Conference conference = mConversation.getCurrentCall();
         if (conference == null || !conference.isOnGoing()) {
-            mConversationFacade.sendTextMessage(mAccountId, mConversation, mContactRingId, message);
+            mConversationFacade.sendTextMessage(mAccountId, mConversation, mContactRingId, message).subscribe();
         } else {
             mConversationFacade.sendTextMessage(mConversation, conference, message);
         }
@@ -234,7 +227,7 @@ public class ConversationPresenter extends RootPresenter<ConversationView> {
     }
 
     public void sendFile(File file) {
-        mConversationFacade.sendFile(mAccountId, mContactRingId, file);
+        mConversationFacade.sendFile(mAccountId, mContactRingId, file).subscribe();
     }
 
     public void downloadFile(final DataTransfer transfer, final File dest) {
@@ -263,25 +256,29 @@ public class ConversationPresenter extends RootPresenter<ConversationView> {
         getView().shareFile(path);
     }
 
+    public void openFile(DataTransfer file) {
+        File path = getDeviceRuntimeService().getConversationPath(file.getPeerId(), file.getStoragePath());
+        getView().openFile(path);
+    }
+
     public void deleteFile(DataTransfer transfer) {
         mConversationFacade.deleteFile(transfer);
     }
 
     public void sendTrustRequest() {
-        VCard vCard = mVCardService.loadSmallVCard(mAccountId);
-        mAccountService.sendTrustRequest(mAccountId, mContactRingId.getRawRingId(), Blob.fromString(VCardUtils.vcardToString(vCard)));
-
-        CallContact contact = mContactService.findContact(mAccountId, mContactRingId);
-        if (contact == null) {
-            Log.e(TAG, "sendTrustRequest: not able to find contact");
-            return;
-        }
-        contact.setStatus(CallContact.Status.REQUEST_SENT);
-    }
-
-    public void blockContact() {
-        mAccountService.removeContact(mAccountId, mContactRingId.getHost(), true);
-        getView().goToHome();
+        final String accountId = mAccountId;
+        final Uri contactId = mContactRingId;
+        mVCardService.loadSmallVCard(accountId, VCardService.MAX_SIZE_REQUEST)
+                .subscribeOn(Schedulers.computation())
+                .subscribe(vCard -> {
+                    mAccountService.sendTrustRequest(accountId, contactId.getRawRingId(), Blob.fromString(VCardUtils.vcardToString(vCard)));
+                    CallContact contact = mContactService.findContact(mAccountService.getAccount(accountId), contactId);
+                    if (contact == null) {
+                        Log.e(TAG, "sendTrustRequest: not able to find contact");
+                        return;
+                    }
+                    contact.setStatus(CallContact.Status.REQUEST_SENT);
+                });
     }
 
     public void clickOnGoingPane() {
@@ -315,11 +312,6 @@ public class ConversationPresenter extends RootPresenter<ConversationView> {
         }
     }
 
-    public void deleteConversation() {
-        mConversationFacade.clearHistory(mAccountId, mContactRingId).subscribe();
-        getView().goToHome();
-    }
-
     private void updateOngoingCallView() {
         Conference conf = mConversation.getCurrentCall();
         if (conf != null && conf.getState() != SipCall.State.INACTIVE) {
@@ -330,16 +322,6 @@ public class ConversationPresenter extends RootPresenter<ConversationView> {
         getView().scrollToEnd();
     }
 
-    private TrustRequest getIncomingTrustRequests() {
-        if (mAccountService != null) {
-            Account acc = mAccountService.getAccount(mAccountId);
-            if (acc != null) {
-                return acc.getRequest(mContactRingId.getHost());
-            }
-        }
-        return null;
-    }
-
     public void onBlockIncomingContactRequest() {
         String accountId = mAccountId == null ? mAccountService.getCurrentAccount().getAccountID() : mAccountId;
         mConversationFacade.discardRequest(accountId, mContactRingId);
@@ -373,4 +355,17 @@ public class ConversationPresenter extends RootPresenter<ConversationView> {
         Log.e(TAG, "configureForFileInfoTextMessage: no space left on device");
         getView().displayErrorToast(RingError.NO_SPACE_LEFT);
     }
-}
\ No newline at end of file
+
+    public void setConversationColor(int color) {
+        mCompositeDisposable.add(mConversationSubject
+                .firstElement()
+                .subscribe(conversation -> conversation.setColor(color)));
+    }
+
+    public void cameraPermissionChanged(boolean isGranted) {
+        if (isGranted && mHardwareService.isVideoAvailable()) {
+            mHardwareService.initVideo();
+        }
+    }
+
+}
