@@ -30,6 +30,7 @@
 #include "logger.h"
 #include "manager.h"
 #include "string_utils.h"
+#include "system_codec_container.h"
 
 #ifdef RING_ACCEL
 #include "video/accel.h"
@@ -49,7 +50,7 @@ extern "C" {
 // Define following line if you need to debug libav SDP
 //#define DEBUG_SDP 1
 
-namespace ring {
+namespace jami {
 
 MediaEncoder::MediaEncoder()
     : outputCtx_(avformat_alloc_context())
@@ -75,74 +76,44 @@ MediaEncoder::~MediaEncoder()
 }
 
 void
-MediaEncoder::setDeviceOptions(const DeviceParams& args)
+MediaEncoder::setOptions(const MediaStream& opts)
 {
-    device_ = args;
-    // Make sure width and height are even (required by x264)
-    // This is especially for image/gif streaming, as video files and cameras usually have even resolutions
-    device_.width -= device_.width % 2;
-    device_.height -= device_.height % 2;
-    if (device_.width)
-        libav_utils::setDictValue(&options_, "width", ring::to_string(device_.width));
-    if (device_.height)
-        libav_utils::setDictValue(&options_, "height", ring::to_string(device_.height));
-    if (not device_.framerate)
-        device_.framerate = 30;
-    libav_utils::setDictValue(&options_, "framerate", ring::to_string(device_.framerate.real()));
+    if (!opts.isValid()) {
+        JAMI_ERR() << "Invalid options";
+        return;
+    }
+
+    if (opts.isVideo) {
+        videoOpts_ = opts;
+        // Make sure width and height are even (required by x264)
+        // This is especially for image/gif streaming, as video files and cameras usually have even resolutions
+        videoOpts_.width -= videoOpts_.width % 2;
+        videoOpts_.height -= videoOpts_.height % 2;
+        if (not videoOpts_.frameRate)
+            videoOpts_.frameRate = 30;
+    } else {
+        audioOpts_ = opts;
+    }
 }
 
 void
 MediaEncoder::setOptions(const MediaDescription& args)
 {
-    codec_ = args.codec;
-
-    libav_utils::setDictValue(&options_, "payload_type", ring::to_string(args.payload_type));
-    libav_utils::setDictValue(&options_, "max_rate", ring::to_string(args.codec->bitrate));
-    libav_utils::setDictValue(&options_, "crf", ring::to_string(args.codec->quality));
-
-    if (args.codec->systemCodecInfo.mediaType == MEDIA_AUDIO) {
-        auto accountAudioCodec = std::static_pointer_cast<AccountAudioCodecInfo>(args.codec);
-        if (accountAudioCodec->audioformat.sample_rate)
-            libav_utils::setDictValue(&options_, "sample_rate",
-                        ring::to_string(accountAudioCodec->audioformat.sample_rate));
-
-        if (accountAudioCodec->audioformat.nb_channels)
-            libav_utils::setDictValue(&options_, "channels",
-                        ring::to_string(accountAudioCodec->audioformat.nb_channels));
-
-        if (accountAudioCodec->audioformat.sample_rate && accountAudioCodec->audioformat.nb_channels)
-            libav_utils::setDictValue(&options_, "frame_size",
-                        ring::to_string(static_cast<unsigned>(0.02 * accountAudioCodec->audioformat.sample_rate)));
-    }
+    libav_utils::setDictValue(&options_, "payload_type", std::to_string(args.payload_type));
+    libav_utils::setDictValue(&options_, "max_rate", std::to_string(args.codec->bitrate));
+    libav_utils::setDictValue(&options_, "crf", std::to_string(args.codec->quality));
 
     if (not args.parameters.empty())
         libav_utils::setDictValue(&options_, "parameters", args.parameters);
 }
 
 void
-MediaEncoder::setOptions(std::map<std::string, std::string> options)
+MediaEncoder::setMetadata(const std::string& title, const std::string& description)
 {
-    const auto& titleIt = options.find("title");
-    if (titleIt != options.end() and not titleIt->second.empty())
-        libav_utils::setDictValue(&outputCtx_->metadata, titleIt->first, titleIt->second);
-    const auto& descIt = options.find("description");
-    if (descIt != options.end() and not descIt->second.empty())
-        libav_utils::setDictValue(&outputCtx_->metadata, descIt->first, descIt->second);
-
-    auto bitrate = SystemCodecInfo::DEFAULT_MAX_BITRATE;
-    auto quality = SystemCodecInfo::DEFAULT_CODEC_QUALITY;
-    // ensure all options retrieved later on are in options_ (insert does nothing if key exists)
-    options.insert({"max_rate", ring::to_string(bitrate)});
-    options.insert({"crf", ring::to_string(quality)});
-    options.insert({"sample_rate", "8000"});
-    options.insert({"channels", "2"});
-    int sampleRate = atoi(options["sample_rate"].c_str());
-    options.insert({"frame_size", ring::to_string(static_cast<unsigned>(0.02*sampleRate))});
-    options.insert({"width", "320"});
-    options.insert({"height", "240"});
-    options.insert({"framerate", "30.00"});
-    for (const auto& it : options)
-        libav_utils::setDictValue(&options_, it.first, it.second);
+    if (not title.empty())
+        libav_utils::setDictValue(&outputCtx_->metadata, "title", title);
+    if (not description.empty())
+        libav_utils::setDictValue(&outputCtx_->metadata, "description", description);
 }
 
 void
@@ -166,7 +137,10 @@ MediaEncoder::getLastSeqValue()
 std::string
 MediaEncoder::getEncoderName() const
 {
-    return encoders_[currentStreamIdx_]->codec->name;
+    if (videoOpts_.isValid())
+        return videoCodec_;
+    else
+        return audioCodec_;
 }
 
 void
@@ -185,6 +159,32 @@ MediaEncoder::openOutput(const std::string& filename, const std::string& format)
 
 int
 MediaEncoder::addStream(const SystemCodecInfo& systemCodecInfo)
+{
+    if (systemCodecInfo.mediaType == MEDIA_AUDIO) {
+        audioCodec_ = systemCodecInfo.name;
+        return initStream(systemCodecInfo, nullptr);
+    } else {
+        videoCodec_ = systemCodecInfo.name;
+        // TODO only support 1 audio stream and 1 video stream per encoder
+        if (audioOpts_.isValid())
+            return 1; // stream will be added to AVFormatContext after audio stream
+        else
+            return 0; // only a video stream
+    }
+}
+
+int
+MediaEncoder::initStream(const std::string& codecName, AVBufferRef* framesCtx)
+{
+    const auto codecInfo = getSystemCodecContainer()->searchCodecByName(codecName, MEDIA_ALL);
+    if (codecInfo)
+        return initStream(*codecInfo, framesCtx);
+    else
+        return -1;
+}
+
+int
+MediaEncoder::initStream(const SystemCodecInfo& systemCodecInfo, AVBufferRef* framesCtx)
 {
     AVCodec* outputCodec = nullptr;
     AVCodecContext* encoderCtx = nullptr;
@@ -192,11 +192,12 @@ MediaEncoder::addStream(const SystemCodecInfo& systemCodecInfo)
     if (systemCodecInfo.mediaType == MEDIA_VIDEO) {
         if (enableAccel_) {
             if (accel_ = video::HardwareAccel::setupEncoder(
-                static_cast<AVCodecID>(systemCodecInfo.avcodecId), device_.width, device_.height)) {
+                static_cast<AVCodecID>(systemCodecInfo.avcodecId),
+                videoOpts_.width, videoOpts_.height, framesCtx)) {
                 outputCodec = avcodec_find_encoder_by_name(accel_->getCodecName().c_str());
             }
         } else {
-            RING_WARN() << "Hardware encoding disabled";
+            JAMI_WARN() << "Hardware encoding disabled";
         }
     }
 #endif
@@ -211,7 +212,7 @@ MediaEncoder::addStream(const SystemCodecInfo& systemCodecInfo)
         else
             outputCodec = avcodec_find_encoder(static_cast<AVCodecID>(systemCodecInfo.avcodecId));
         if (!outputCodec) {
-            RING_ERR("Encoder \"%s\" not found!", systemCodecInfo.name.c_str());
+            JAMI_ERR("Encoder \"%s\" not found!", systemCodecInfo.name.c_str());
             throw MediaEncoderException("No output encoder");
         }
     }
@@ -221,7 +222,7 @@ MediaEncoder::addStream(const SystemCodecInfo& systemCodecInfo)
 
 #ifdef RING_ACCEL
     if (accel_) {
-        accel_->setDetails(encoderCtx, &options_);
+        accel_->setDetails(encoderCtx);
         encoderCtx->opaque = accel_.get();
     }
 #endif
@@ -235,6 +236,13 @@ MediaEncoder::addStream(const SystemCodecInfo& systemCodecInfo)
         auto profileLevelId = libav_utils::getDictValue(options_, "parameters");
         extractProfileLevelID(profileLevelId, encoderCtx);
 #ifdef RING_ACCEL
+#ifdef ENABLE_VIDEOTOOLBOX
+        if (accel_) {
+            maxBitrate = 2000 * std::atoi(libav_utils::getDictValue(options_, "max_rate"));
+            bufSize = 2 * maxBitrate;
+            crf = 20;
+        }
+#endif
         if (accel_)
             // limit the bitrate else it will easily go up to a few MiB/s
             encoderCtx->bit_rate = maxBitrate;
@@ -245,7 +253,7 @@ MediaEncoder::addStream(const SystemCodecInfo& systemCodecInfo)
         // Streaming => VBV (constrained encoding) + CRF (Constant Rate Factor)
         if (crf == SystemCodecInfo::DEFAULT_NO_QUALITY)
             crf = 30; // good value for H264-720p@30
-        RING_DBG("H264 encoder setup: crf=%u, maxrate=%u, bufsize=%u", crf, maxBitrate, bufSize);
+        JAMI_DBG("H264 encoder setup: crf=%u, maxrate=%u, bufsize=%u", crf, maxBitrate, bufSize);
 
         av_opt_set_int(encoderCtx, "crf", crf, AV_OPT_SEARCH_CHILDREN);
         encoderCtx->rc_buffer_size = bufSize;
@@ -273,9 +281,9 @@ MediaEncoder::addStream(const SystemCodecInfo& systemCodecInfo)
         encoderCtx->bit_rate = maxBitrate;
         if (crf != SystemCodecInfo::DEFAULT_NO_QUALITY) {
             av_opt_set_int(encoderCtx, "crf", crf, AV_OPT_SEARCH_CHILDREN);
-            RING_DBG("Using quality factor %d", crf);
+            JAMI_DBG("Using quality factor %d", crf);
         } else {
-            RING_DBG("Using Max bitrate %d", maxBitrate);
+            JAMI_DBG("Using Max bitrate %d", maxBitrate);
         }
     } else if (systemCodecInfo.avcodecId == AV_CODEC_ID_MPEG4) {
         // For MPEG4 :
@@ -283,11 +291,11 @@ MediaEncoder::addStream(const SystemCodecInfo& systemCodecInfo)
         // Use CBR (set bitrate)
         encoderCtx->rc_buffer_size = maxBitrate;
         encoderCtx->bit_rate = encoderCtx->rc_min_rate = encoderCtx->rc_max_rate =  maxBitrate;
-        RING_DBG("Using Max bitrate %d", maxBitrate);
+        JAMI_DBG("Using Max bitrate %d", maxBitrate);
     } else if (systemCodecInfo.avcodecId == AV_CODEC_ID_H263) {
         encoderCtx->bit_rate = encoderCtx->rc_max_rate =  maxBitrate;
         encoderCtx->rc_buffer_size = maxBitrate;
-        RING_DBG("Using Max bitrate %d", maxBitrate);
+        JAMI_DBG("Using Max bitrate %d", maxBitrate);
     }
 
     // add video stream to outputformat context
@@ -308,7 +316,7 @@ MediaEncoder::addStream(const SystemCodecInfo& systemCodecInfo)
 #endif
     // framerate is not copied from encoderCtx to stream
     stream->avg_frame_rate = encoderCtx->framerate;
-#ifdef RING_VIDEO
+#ifdef ENABLE_VIDEO
     if (systemCodecInfo.mediaType == MEDIA_VIDEO) {
         // allocate buffers for both scaled (pre-encoder) and encoded frames
         const int width = encoderCtx->width;
@@ -331,16 +339,16 @@ MediaEncoder::addStream(const SystemCodecInfo& systemCodecInfo)
         scaledFrameBuffer_.reserve(scaledFrameBufferSize_);
         scaledFrame_.setFromMemory(scaledFrameBuffer_.data(), format, width, height);
     }
-#endif // RING_VIDEO
+#endif // ENABLE_VIDEO
 
     return stream->index;
 }
 
 void
-MediaEncoder::setIOContext(AVIOContext* ioctx)
+MediaEncoder::openIOContext()
 {
-    if (ioctx) {
-        outputCtx_->pb = ioctx;
+    if (ioCtx_) {
+        outputCtx_->pb = ioCtx_;
         outputCtx_->packet_size = outputCtx_->pb->buffer_size;
     } else {
         int ret = 0;
@@ -352,7 +360,7 @@ MediaEncoder::setIOContext(AVIOContext* ioctx)
         if (!(outputCtx_->oformat->flags & AVFMT_NOFILE)) {
             if ((ret = avio_open(&outputCtx_->pb, filename, AVIO_FLAG_WRITE)) < 0) {
                 std::stringstream ss;
-                ss << "Could not set IO context for '" << filename << "': " << libav_utils::getError(ret);
+                ss << "Could not open IO context for '" << filename << "': " << libav_utils::getError(ret);
                 throw MediaEncoderException(ss.str().c_str());
             }
         }
@@ -362,8 +370,10 @@ MediaEncoder::setIOContext(AVIOContext* ioctx)
 void
 MediaEncoder::startIO()
 {
+    if (!outputCtx_->pb)
+        openIOContext();
     if (avformat_write_header(outputCtx_, options_ ? &options_ : nullptr)) {
-        RING_ERR("Could not write header for output file... check codec parameters");
+        JAMI_ERR("Could not write header for output file... check codec parameters");
         throw MediaEncoderException("Failed to write output file header");
     }
 
@@ -372,31 +382,67 @@ MediaEncoder::startIO()
 #else
     av_dump_format(outputCtx_, 0, outputCtx_->filename, 1);
 #endif
+    initialized_ = true;
 }
 
-#ifdef RING_VIDEO
+#ifdef ENABLE_VIDEO
 int
-MediaEncoder::encode(VideoFrame& input, bool is_keyframe,
-                     int64_t frame_number)
+MediaEncoder::encode(VideoFrame& input, bool is_keyframe, int64_t frame_number)
 {
-    /* Prepare a frame suitable to our encoder frame format,
-     * keeping also the input aspect ratio.
-     */
-    libav_utils::fillWithBlack(scaledFrame_.pointer());
+    if (!initialized_) {
+        initStream(videoCodec_, input.pointer()->hw_frames_ctx);
+        startIO();
+    }
 
-    scaler_.scale_with_aspect(input, scaledFrame_);
+    AVFrame* frame;
+#ifdef RING_ACCEL
+    auto desc = av_pix_fmt_desc_get(static_cast<AVPixelFormat>(input.format()));
+    bool isHardware = desc && (desc->flags & AV_PIX_FMT_FLAG_HWACCEL);
+#ifdef ENABLE_VIDEOTOOLBOX
+    //Videotoolbox handles frames allocations itself and do not need creating frame context manually.
+    //Now videotoolbox supports only fully accelerated pipeline
+    bool isVideotoolbox = static_cast<AVPixelFormat>(input.format()) == AV_PIX_FMT_VIDEOTOOLBOX;
+    if (accel_ &&  isVideotoolbox) {
+        // Fully accelerated pipeline, skip main memory
+        frame = input.pointer();
+    } else {
+#else
+    std::unique_ptr<VideoFrame> framePtr;
+    if (accel_ && accel_->isLinked()) {
+        // Fully accelerated pipeline, skip main memory
+        frame = input.pointer();
+    } else if (isHardware) {
+        // Hardware decoded frame, transfer back to main memory
+        // Transfer to GPU if we have a hardware encoder
+        AVPixelFormat pix = (accel_ ? accel_->getSoftwareFormat() : AV_PIX_FMT_YUV420P);
+        framePtr = video::HardwareAccel::transferToMainMemory(input, pix);
+        if (accel_)
+            framePtr = accel_->transfer(*framePtr);
+        frame = framePtr->pointer();
+    } else if (accel_) {
+        // Software decoded frame with a hardware encoder, convert to accepted format first
+        auto pix = accel_->getSoftwareFormat();
+        if (input.format() != pix) {
+            framePtr = scaler_.convertFormat(input, pix);
+            framePtr = accel_->transfer(*framePtr);
+        } else {
+            framePtr = accel_->transfer(input);
+        }
+        frame = framePtr->pointer();
+    } else {
+#endif //ENABLE_VIDEOTOOLBOX
+#endif
+        libav_utils::fillWithBlack(scaledFrame_.pointer());
+        scaler_.scale_with_aspect(input, scaledFrame_);
+        frame = scaledFrame_.pointer();
+#ifdef RING_ACCEL
+    }
+#endif
 
-    // Copy frame so the VideoScaler can still use the software frame (input)
-    VideoFrame copy;
-    copy.copyFrom(scaledFrame_);
-
-    auto frame = copy.pointer();
     AVCodecContext* enc = encoders_[currentStreamIdx_];
-    // ideally, time base is the inverse of framerate, but this may not always be the case
-    if (enc->framerate.num == enc->time_base.den && enc->framerate.den == enc->time_base.num)
-        frame->pts = frame_number;
-    else
-        frame->pts = (frame_number / (rational<int64_t>(enc->framerate) * rational<int64_t>(enc->time_base))).real<int64_t>();
+    frame->pts = frame_number;
+    if (enc->framerate.num != enc->time_base.den || enc->framerate.den != enc->time_base.num)
+        frame->pts /= (rational<int64_t>(enc->framerate) * rational<int64_t>(enc->time_base)).real<int64_t>();
 
     if (is_keyframe) {
         frame->pict_type = AV_PICTURE_TYPE_I;
@@ -406,26 +452,20 @@ MediaEncoder::encode(VideoFrame& input, bool is_keyframe,
         frame->key_frame = 0;
     }
 
-#ifdef RING_ACCEL
-    // NOTE needs to be at same scope as call to encode
-    std::unique_ptr<VideoFrame> framePtr;
-    if (accel_) {
-        framePtr = accel_->transfer(copy);
-        if (!framePtr) {
-            RING_ERR() << "Hardware encoding failure";
-            return -1;
-        }
-        frame = framePtr->pointer();
-    }
-#endif
-
     return encode(frame, currentStreamIdx_);
 }
-#endif // RING_VIDEO
+#endif // ENABLE_VIDEO
 
 int
 MediaEncoder::encodeAudio(AudioFrame& frame)
 {
+    if (!initialized_) {
+        // Initialize on first video frame, or first audio frame if no video stream
+        if (not videoOpts_.isValid())
+            startIO();
+        else
+            return 0;
+    }
     frame.pointer()->pts = sent_samples;
     sent_samples += frame.pointer()->nb_samples;
     encode(frame.pointer(), currentStreamIdx_);
@@ -435,6 +475,18 @@ MediaEncoder::encodeAudio(AudioFrame& frame)
 int
 MediaEncoder::encode(AVFrame* frame, int streamIdx)
 {
+    if (not frame)
+        return -1;
+    if (!initialized_) {
+        // Initialize on first video frame, or first audio frame if no video stream
+        bool isVideo = (frame->width > 0 && frame->height > 0);
+        if (isVideo or not videoOpts_.isValid()) {
+            streamIdx = initStream(videoCodec_, frame->hw_frames_ctx);
+            startIO();
+        } else {
+            return 0;
+        }
+    }
     int ret = 0;
     AVCodecContext* encoderCtx = encoders_[streamIdx];
     AVPacket pkt;
@@ -451,7 +503,7 @@ MediaEncoder::encode(AVFrame* frame, int streamIdx)
         if (ret == AVERROR(EAGAIN))
             break;
         if (ret < 0 && ret != AVERROR_EOF) { // we still want to write our frame on EOF
-            RING_ERR() << "Failed to encode frame: " << libav_utils::getError(ret);
+            JAMI_ERR() << "Failed to encode frame: " << libav_utils::getError(ret);
             return ret;
         }
 
@@ -468,20 +520,26 @@ MediaEncoder::encode(AVFrame* frame, int streamIdx)
 bool
 MediaEncoder::send(AVPacket& pkt, int streamIdx)
 {
+    if (!initialized_) {
+        streamIdx = initStream(videoCodec_, nullptr);
+        startIO();
+    }
     if (streamIdx < 0)
         streamIdx = currentStreamIdx_;
-    auto encoderCtx = encoders_[streamIdx];
-    pkt.stream_index = streamIdx;
-    if (pkt.pts != AV_NOPTS_VALUE)
-        pkt.pts = av_rescale_q(pkt.pts, encoderCtx->time_base,
-                               outputCtx_->streams[streamIdx]->time_base);
-    if (pkt.dts != AV_NOPTS_VALUE)
-        pkt.dts = av_rescale_q(pkt.dts, encoderCtx->time_base,
-                               outputCtx_->streams[streamIdx]->time_base);
+    if (streamIdx >= 0 and streamIdx < encoders_.size()) {
+        auto encoderCtx = encoders_[streamIdx];
+        pkt.stream_index = streamIdx;
+        if (pkt.pts != AV_NOPTS_VALUE)
+            pkt.pts = av_rescale_q(pkt.pts, encoderCtx->time_base,
+                                outputCtx_->streams[streamIdx]->time_base);
+        if (pkt.dts != AV_NOPTS_VALUE)
+            pkt.dts = av_rescale_q(pkt.dts, encoderCtx->time_base,
+                                outputCtx_->streams[streamIdx]->time_base);
+    }
     // write the compressed frame
     auto ret = av_write_frame(outputCtx_, &pkt);
     if (ret < 0) {
-        RING_ERR() << "av_write_frame failed: " << libav_utils::getError(ret);
+        JAMI_ERR() << "av_write_frame failed: " << libav_utils::getError(ret);
     }
     return ret >= 0;
 }
@@ -492,7 +550,7 @@ MediaEncoder::flush()
     int ret = 0;
     for (size_t i = 0; i < outputCtx_->nb_streams; ++i) {
         if (encode(nullptr, i) < 0) {
-            RING_ERR() << "Could not flush stream #" << i;
+            JAMI_ERR() << "Could not flush stream #" << i;
             ret |= 1u << i; // provide a way for caller to know which streams failed
         }
     }
@@ -519,7 +577,7 @@ MediaEncoder::print_sdp()
         result += line + "\n";
     }
 #ifdef DEBUG_SDP
-    RING_DBG("Sending SDP:\n%s", result.c_str());
+    JAMI_DBG("Sending SDP:\n%s", result.c_str());
 #endif
     return result;
 }
@@ -532,38 +590,19 @@ MediaEncoder::prepareEncoderContext(AVCodec* outputCodec, bool is_video)
     auto encoderName = outputCodec->name; // guaranteed to be non null if AVCodec is not null
 
     encoderCtx->thread_count = std::min(std::thread::hardware_concurrency(), is_video ? 16u : 4u);
-    RING_DBG("[%s] Using %d threads", encoderName, encoderCtx->thread_count);
+    JAMI_DBG("[%s] Using %d threads", encoderName, encoderCtx->thread_count);
 
     if (is_video) {
         // resolution must be a multiple of two
-        if (device_.width && device_.height) {
-            encoderCtx->width = device_.width;
-            encoderCtx->height = device_.height;
-        } else {
-            encoderCtx->width = std::atoi(libav_utils::getDictValue(options_, "width"));
-            encoderCtx->height = std::atoi(libav_utils::getDictValue(options_, "height"));
-        }
+        encoderCtx->width = videoOpts_.width;
+        encoderCtx->height = videoOpts_.height;
 
         // satisfy ffmpeg: denominator must be 16bit or less value
         // time base = 1/FPS
-        if (device_.framerate) {
-            av_reduce(&encoderCtx->framerate.num, &encoderCtx->framerate.den,
-                      device_.framerate.numerator(), device_.framerate.denominator(),
-                      (1U << 16) - 1);
-            encoderCtx->time_base = av_inv_q(encoderCtx->framerate);
-        } else {
-            // get from options_, else default to 30 fps
-            auto v = libav_utils::getDictValue(options_, "framerate");
-            AVRational framerate = AVRational{30, 1};
-            if (v)
-                av_parse_ratio_quiet(&framerate, v, 120);
-            if (framerate.den == 0)
-                framerate.den = 1;
-            av_reduce(&encoderCtx->framerate.num, &encoderCtx->framerate.den,
-                      framerate.num, framerate.den,
-                      (1U << 16) - 1);
-            encoderCtx->time_base = av_inv_q(encoderCtx->framerate);
-        }
+        av_reduce(&encoderCtx->framerate.num, &encoderCtx->framerate.den,
+                  videoOpts_.frameRate.numerator(), videoOpts_.frameRate.denominator(),
+                  (1U << 16) - 1);
+        encoderCtx->time_base = av_inv_q(encoderCtx->framerate);
 
         // emit one intra frame every gop_size frames
         encoderCtx->max_b_frames = 0;
@@ -580,36 +619,21 @@ MediaEncoder::prepareEncoderContext(AVCodec* outputCodec, bool is_video)
         // encoderCtx->flags |= AV_CODEC_FLAG_GLOBAL_HEADER;
     } else {
         encoderCtx->sample_fmt = AV_SAMPLE_FMT_S16;
-        auto v = libav_utils::getDictValue(options_, "sample_rate");
-        if (v) {
-            encoderCtx->sample_rate = atoi(v);
-            encoderCtx->time_base = AVRational{1, encoderCtx->sample_rate};
+        encoderCtx->sample_rate = std::max(8000, audioOpts_.sampleRate);
+        encoderCtx->time_base = AVRational{1, encoderCtx->sample_rate};
+        if (audioOpts_.nbChannels > 2 || audioOpts_.nbChannels < 1) {
+            encoderCtx->channels = std::max(std::min(audioOpts_.nbChannels, 1), 2);
+            JAMI_ERR() << "[" << encoderName << "] Clamping invalid channel count: "
+                << audioOpts_.nbChannels << " -> " << encoderCtx->channels;
         } else {
-            RING_WARN("[%s] No sample rate set", encoderName);
-            encoderCtx->sample_rate = 8000;
+            encoderCtx->channels = audioOpts_.nbChannels;
         }
-
-        v = libav_utils::getDictValue(options_, "channels");
-        if (v) {
-            auto c = std::atoi(v);
-            if (c > 2 or c < 1) {
-                RING_WARN("[%s] Clamping invalid channel value %d", encoderName, c);
-                c = 1;
-            }
-            encoderCtx->channels = c;
+        encoderCtx->channel_layout = av_get_default_channel_layout(encoderCtx->channels);
+        if (audioOpts_.frameSize) {
+            encoderCtx->frame_size = audioOpts_.frameSize;
+            JAMI_DBG() << "[" << encoderName << "] Frame size " << encoderCtx->frame_size;
         } else {
-            RING_WARN("[%s] Channels not set", encoderName);
-            encoderCtx->channels = 1;
-        }
-
-        encoderCtx->channel_layout = encoderCtx->channels == 2 ? AV_CH_LAYOUT_STEREO : AV_CH_LAYOUT_MONO;
-
-        v = libav_utils::getDictValue(options_, "frame_size");
-        if (v) {
-            encoderCtx->frame_size = atoi(v);
-            RING_DBG("[%s] Frame size %d", encoderName, encoderCtx->frame_size);
-        } else {
-            RING_WARN("[%s] Frame size not set", encoderName);
+            JAMI_WARN() << "[" << encoderName << "] Frame size not set";
         }
     }
 
@@ -621,10 +645,10 @@ MediaEncoder::forcePresetX264(AVCodecContext* encoderCtx)
 {
     const char *speedPreset = "ultrafast";
     if (av_opt_set(encoderCtx, "preset", speedPreset, AV_OPT_SEARCH_CHILDREN))
-        RING_WARN("Failed to set x264 preset '%s'", speedPreset);
+        JAMI_WARN("Failed to set x264 preset '%s'", speedPreset);
     const char *tune = "zerolatency";
     if (av_opt_set(encoderCtx, "tune", tune, AV_OPT_SEARCH_CHILDREN))
-        RING_WARN("Failed to set x264 tune '%s'", tune);
+        JAMI_WARN("Failed to set x264 tune '%s'", tune);
 }
 
 void
@@ -673,13 +697,16 @@ MediaEncoder::extractProfileLevelID(const std::string &parameters,
                 ctx->profile |= FF_PROFILE_H264_INTRA;
             break;
     }
-    RING_DBG("Using profile %x and level %d", ctx->profile, ctx->level);
+    JAMI_DBG("Using profile %x and level %d", ctx->profile, ctx->level);
 }
 
 bool
-MediaEncoder::useCodec(const ring::AccountCodecInfo* codec) const noexcept
+MediaEncoder::useCodec(const jami::AccountCodecInfo* codec) const noexcept
 {
-    return codec_.get() == codec;
+    if (codec->systemCodecInfo.mediaType == MEDIA_VIDEO)
+        return videoCodec_ == codec->systemCodecInfo.name;
+    else
+        return audioCodec_ == codec->systemCodecInfo.name;
 }
 
 #ifdef RING_ACCEL
@@ -699,10 +726,7 @@ MediaEncoder::enableAccel(bool enableAccel)
 unsigned
 MediaEncoder::getStreamCount() const
 {
-    if (outputCtx_)
-        return outputCtx_->nb_streams;
-    else
-        return 0;
+    return (audioOpts_.isValid() + videoOpts_.isValid());
 }
 
 MediaStream
@@ -735,31 +759,30 @@ MediaEncoder::readConfig(AVDictionary** dict, AVCodecContext* encoderCtx)
             std::ifstream file(path);
             file >> root;
             if (!root.isObject()) {
-                RING_ERR() << "Invalid encoder configuration: root is not an object";
+                JAMI_ERR() << "Invalid encoder configuration: root is not an object";
                 return;
             }
             const auto& config = root[name];
             if (config.isNull()) {
-                RING_WARN() << "Encoder '" << name << "' not found in configuration file";
+                JAMI_WARN() << "Encoder '" << name << "' not found in configuration file";
                 return;
             }
             if (!config.isObject()) {
-                RING_ERR() << "Invalid encoder configuration: '" << name << "' is not an object";
+                JAMI_ERR() << "Invalid encoder configuration: '" << name << "' is not an object";
                 return;
             }
             // If users want to change these, they should use the settings page.
-            std::vector<std::string> ignoredKeys = { "width", "height", "framerate", "sample_rate", "channels", "frame_size", "parameters" };
             for (Json::Value::const_iterator it = config.begin(); it != config.end(); ++it) {
                 Json::Value v = *it;
                 if (!it.key().isConvertibleTo(Json::ValueType::stringValue)
                     || !v.isConvertibleTo(Json::ValueType::stringValue)) {
-                    RING_ERR() << "Invalid configuration for '" << name << "'";
+                    JAMI_ERR() << "Invalid configuration for '" << name << "'";
                     return;
                 }
                 const auto& key = it.key().asString();
                 const auto& value = v.asString();
                 // provides a way to override all AVCodecContext fields MediaEncoder sets
-                if (std::find(ignoredKeys.cbegin(), ignoredKeys.cend(), key) != ignoredKeys.cend())
+                if (key == "parameters") // Used by MediaEncoder for profile-level-id, ignore
                     continue;
                 else if (value.empty())
                     libav_utils::setDictValue(dict, key, nullptr);
@@ -783,9 +806,9 @@ MediaEncoder::readConfig(AVDictionary** dict, AVCodecContext* encoderCtx)
                     libav_utils::setDictValue(dict, key, value);
             }
         } catch (const Json::Exception& e) {
-            RING_ERR() << "Failed to load encoder configuration file: " << e.what();
+            JAMI_ERR() << "Failed to load encoder configuration file: " << e.what();
         }
     }
 }
 
-} // namespace ring
+} // namespace jami
