@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2016 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2019 Savoir-faire Linux Inc.
  *
  *  Author: Tristan Matthews <tristan.matthews@savoirfairelinux.com>
  *  Author: Guillaume Roguez <Guillaume.Roguez@savoirfairelinux.com>
@@ -31,6 +31,7 @@
 #include "logger.h"
 #include "string_utils.h"
 #include "call.h"
+#include "conference.h"
 
 #include "account_const.h"
 
@@ -40,26 +41,47 @@
 #include <thread>
 #include <chrono>
 
-namespace ring { namespace video {
+namespace jami { namespace video {
 
-using std::map;
 using std::string;
 
-constexpr static auto NEWPARAMS_TIMEOUT = std::chrono::milliseconds(1000);
+// how long (in seconds) to wait before rechecking for packet loss
+static constexpr auto RTCP_PACKET_LOSS_INTERVAL = std::chrono::milliseconds(1000);
 
 VideoRtpSession::VideoRtpSession(const string &callID,
                                  const DeviceParams& localVideoParams) :
     RtpSession(callID), localVideoParams_(localVideoParams)
     , lastRTCPCheck_(std::chrono::system_clock::now())
     , lastLongRTCPCheck_(std::chrono::system_clock::now())
-    , rtcpCheckerThread_(std::bind(&VideoRtpSession::setupRtcpChecker, this),
-            std::bind(&VideoRtpSession::processRtcpChecker, this),
-            std::bind(&VideoRtpSession::cleanupRtcpChecker, this))
-{}
+    , videoBitrateInfo_ {}
+    , rtcpCheckerThread_([] { return true; },
+            [this]{ processRtcpChecker(); },
+            []{})
+    , packetLossThread_([] { return true; },
+            [this]{ processPacketLoss(); },
+            []{})
+{
+    setupVideoBitrateInfo(); // reset bitrate
+}
 
 VideoRtpSession::~VideoRtpSession()
 { stop(); }
 
+/// Setup internal VideoBitrateInfo structure from media descriptors.
+///
+void
+VideoRtpSession::updateMedia(const MediaDescription& send, const MediaDescription& receive)
+{
+    BaseType::updateMedia(send, receive);
+    setupVideoBitrateInfo();
+}
+
+void
+VideoRtpSession::setRequestKeyFrameCallback(std::function<void(void)> cb)
+{
+    requestKeyFrameCallback_ = std::move(cb);
+}
+
 void VideoRtpSession::startSender()
 {
     if (send_.enabled and not send_.holding) {
@@ -68,7 +90,7 @@ void VideoRtpSession::startSender()
                 videoLocal_->detach(sender_.get());
             if (videoMixer_)
                 videoMixer_->detach(sender_.get());
-            RING_WARN("Restarting video sender");
+            JAMI_WARN("Restarting video sender");
         }
 
         if (not conference_) {
@@ -77,16 +99,20 @@ void VideoRtpSession::startSender()
                 auto newParams = input->switchInput(input_);
                 try {
                     if (newParams.valid() &&
-                        newParams.wait_for(NEWPARAMS_TIMEOUT) == std::future_status::ready)
+                        newParams.wait_for(NEWPARAMS_TIMEOUT) == std::future_status::ready) {
                         localVideoParams_ = newParams.get();
-                    else
-                        RING_ERR("No valid new video parameters.");
+                    } else {
+                        JAMI_ERR("No valid new video parameters.");
+                        return;
+                    }
                 } catch (const std::exception& e) {
-                    RING_ERR("Exception during retrieving video parameters: %s",
+                    JAMI_ERR("Exception during retrieving video parameters: %s",
                              e.what());
+                    return;
                 }
             } else {
-                RING_WARN("Can't lock video input");
+                JAMI_WARN("Can't lock video input");
+                return;
             }
         }
 
@@ -99,16 +125,19 @@ void VideoRtpSession::startSender()
             sender_.reset();
             socketPair_->stopSendOp(false);
             sender_.reset(new VideoSender(getRemoteRtpUri(), localVideoParams_,
-                                          send_, *socketPair_, initSeqVal_));
+                                          send_, *socketPair_, initSeqVal_, mtu_));
+            if (changeOrientationCallback_)
+                sender_->setChangeOrientationCallback(changeOrientationCallback_);
+
         } catch (const MediaEncoderException &e) {
-            RING_ERR("%s", e.what());
+            JAMI_ERR("%s", e.what());
             send_.enabled = false;
         }
-        auto codecVideo = std::static_pointer_cast<ring::AccountVideoCodecInfo>(send_.codec);
-        auto isAutoQualityEnabledStr = codecVideo->getCodecSpecifications()[DRing::Account::ConfProperties::CodecInfo::AUTO_QUALITY_ENABLED];
-        if ((not rtcpCheckerThread_.isRunning()) && (isAutoQualityEnabledStr.compare(TRUE_STR) == 0))
+        auto codecVideo = std::static_pointer_cast<jami::AccountVideoCodecInfo>(send_.codec);
+        auto autoQuality = codecVideo->isAutoQualityEnabled;
+        if (autoQuality and not rtcpCheckerThread_.isRunning())
             rtcpCheckerThread_.start();
-        else if ((rtcpCheckerThread_.isRunning()) && (isAutoQualityEnabledStr.compare(FALSE_STR) == 0))
+        else if (not autoQuality and rtcpCheckerThread_.isRunning())
             rtcpCheckerThread_.join();
     }
 }
@@ -129,40 +158,26 @@ VideoRtpSession::restartSender()
 void VideoRtpSession::startReceiver()
 {
     if (receive_.enabled and not receive_.holding) {
-        DeviceParams receiverArgs = {};
-        if (receiveThread_) {
-            RING_WARN("Restarting video receiver");
-            receiverArgs.enableAccel = "0"; // most likely cause of this restart
-        }
+        if (receiveThread_)
+            JAMI_WARN("Restarting video receiver");
         receiveThread_.reset(
-            new VideoReceiveThread(callID_, receive_.receiving_sdp, receiverArgs)
+            new VideoReceiveThread(callID_, receive_.receiving_sdp, mtu_)
         );
-        /* ebail: keyframe requests can lead to timeout if they are not answered.
-         * we decided so to disable them for the moment
-        receiveThread_->setRequestKeyFrameCallback(&SIPVoIPLink::enqueueKeyframeRequest);
-        */
+
+        // XXX keyframe requests can timeout if unanswered
+        receiveThread_->setRequestKeyFrameCallback(requestKeyFrameCallback_);
         receiveThread_->addIOContext(*socketPair_);
         receiveThread_->startLoop();
+        packetLossThread_.start();
     } else {
-        RING_DBG("Video receiving disabled");
+        JAMI_DBG("Video receiving disabled");
         if (receiveThread_)
             receiveThread_->detach(videoMixer_.get());
         receiveThread_.reset();
+        packetLossThread_.join();
     }
 }
 
-void
-VideoRtpSession::restartReceiver()
-{
-    std::lock_guard<std::recursive_mutex> lock(mutex_);
-
-    // ensure that start has been called before restart
-    if (not socketPair_)
-        return;
-
-    startReceiver(); // disable accel
-}
-
 void VideoRtpSession::start(std::unique_ptr<IceSocket> rtp_sock,
                             std::unique_ptr<IceSocket> rtcp_sock)
 {
@@ -186,7 +201,7 @@ void VideoRtpSession::start(std::unique_ptr<IceSocket> rtp_sock,
                                     send_.crypto.getSrtpKeyInfo().c_str());
         }
     } catch (const std::runtime_error& e) {
-        RING_ERR("Socket creation failed: %s", e.what());
+        JAMI_ERR("Socket creation failed: %s", e.what());
         return;
     }
 
@@ -200,7 +215,7 @@ void VideoRtpSession::stop()
 {
     std::lock_guard<std::recursive_mutex> lock(mutex_);
     rtcpCheckerThread_.join();
-
+    packetLossThread_.join();
     if (videoLocal_)
         videoLocal_->detach(sender_.get());
 
@@ -233,6 +248,13 @@ void VideoRtpSession::forceKeyFrame()
         sender_->forceKeyFrame();
 }
 
+void
+VideoRtpSession::setRotation(int rotation)
+{
+    if (receiveThread_)
+        receiveThread_->setRotation(rotation);
+}
+
 void
 VideoRtpSession::setupVideoPipeline()
 {
@@ -249,7 +271,7 @@ VideoRtpSession::setupVideoPipeline()
 void
 VideoRtpSession::setupConferenceVideoPipeline(Conference& conference)
 {
-    RING_DBG("[call:%s] Setup video pipeline on conference %s", callID_.c_str(),
+    JAMI_DBG("[call:%s] Setup video pipeline on conference %s", callID_.c_str(),
              conference.getConfID().c_str());
     videoMixer_ = conference.getVideoMixer();
 
@@ -259,13 +281,13 @@ VideoRtpSession::setupConferenceVideoPipeline(Conference& conference)
             videoLocal_->detach(sender_.get());
         videoMixer_->attach(sender_.get());
     } else
-        RING_WARN("[call:%s] no sender", callID_.c_str());
+        JAMI_WARN("[call:%s] no sender", callID_.c_str());
 
     if (receiveThread_) {
         receiveThread_->enterConference();
         receiveThread_->attach(videoMixer_.get());
     } else
-        RING_WARN("[call:%s] no receiver", callID_.c_str());
+        JAMI_WARN("[call:%s] no receiver", callID_.c_str());
 }
 
 void
@@ -276,7 +298,7 @@ VideoRtpSession::enterConference(Conference* conference)
     exitConference();
 
     conference_ = conference;
-    RING_DBG("[call:%s] enterConference (conf: %s)", callID_.c_str(),
+    JAMI_DBG("[call:%s] enterConference (conf: %s)", callID_.c_str(),
              conference->getConfID().c_str());
 
     if (send_.enabled or receiveThread_) {
@@ -293,7 +315,7 @@ void VideoRtpSession::exitConference()
     if (!conference_)
         return;
 
-    RING_DBG("[call:%s] exitConference (conf: %s)", callID_.c_str(),
+    JAMI_DBG("[call:%s] exitConference (conf: %s)", callID_.c_str(),
              conference_->getConfID().c_str());
 
     if (videoMixer_) {
@@ -315,7 +337,7 @@ void VideoRtpSession::exitConference()
 }
 
 bool
-VideoRtpSession::useCodec(const ring::AccountVideoCodecInfo* codec) const
+VideoRtpSession::useCodec(const jami::AccountVideoCodecInfo* codec) const
 {
     return sender_->useCodec(codec);
 }
@@ -344,7 +366,7 @@ unsigned
 VideoRtpSession::getLowerQuality()
 {
     // if lower quality was stored we return it
-    unsigned quality = videoBitrateInfo_.videoQualityCurrent;
+    unsigned quality = 0;
     while ( not histoQuality_.empty()) {
         quality = histoQuality_.back();
         histoQuality_.pop_back();
@@ -361,7 +383,7 @@ unsigned
 VideoRtpSession::getLowerBitrate()
 {
     // if a lower bitrate was stored we return it
-    unsigned bitrate = videoBitrateInfo_.videoBitrateCurrent;
+    unsigned bitrate = 0;
     while ( not histoBitrate_.empty()) {
         bitrate = histoBitrate_.back();
         histoBitrate_.pop_back();
@@ -477,7 +499,7 @@ VideoRtpSession::adaptQualityAndBitrate()
         storeVideoBitrateInfo();
         const auto& cid = callID_;
 
-        RING_WARN("[%u/%u] packetLostRate=%f -> change quality to %d bitrate to %d",
+        JAMI_WARN("[%u/%u] packetLostRate=%f -> change quality to %d bitrate to %d",
                 videoBitrateInfo_.cptBitrateChecking,
                 videoBitrateInfo_.maxBitrateChecking,
                 packetLostRate,
@@ -490,38 +512,41 @@ VideoRtpSession::adaptQualityAndBitrate()
             });
     }
 }
+
 void
-VideoRtpSession::getVideoBitrateInfo() {
-    auto codecVideo = std::static_pointer_cast<ring::AccountVideoCodecInfo>(send_.codec);
+VideoRtpSession::setupVideoBitrateInfo() {
+    auto codecVideo = std::static_pointer_cast<jami::AccountVideoCodecInfo>(send_.codec);
     if (codecVideo) {
         videoBitrateInfo_ = {
-            (unsigned)(ring::stoi(codecVideo->getCodecSpecifications()[DRing::Account::ConfProperties::CodecInfo::BITRATE])),
-            (unsigned)(ring::stoi(codecVideo->getCodecSpecifications()[DRing::Account::ConfProperties::CodecInfo::MIN_BITRATE])),
-            (unsigned)(ring::stoi(codecVideo->getCodecSpecifications()[DRing::Account::ConfProperties::CodecInfo::MAX_BITRATE])),
-            (unsigned)(ring::stoi(codecVideo->getCodecSpecifications()[DRing::Account::ConfProperties::CodecInfo::QUALITY])),
-            (unsigned)(ring::stoi(codecVideo->getCodecSpecifications()[DRing::Account::ConfProperties::CodecInfo::MIN_QUALITY])),
-            (unsigned)(ring::stoi(codecVideo->getCodecSpecifications()[DRing::Account::ConfProperties::CodecInfo::MAX_QUALITY])),
+            (unsigned)(jami::stoi(codecVideo->getCodecSpecifications()[DRing::Account::ConfProperties::CodecInfo::BITRATE])),
+            (unsigned)(jami::stoi(codecVideo->getCodecSpecifications()[DRing::Account::ConfProperties::CodecInfo::MIN_BITRATE])),
+            (unsigned)(jami::stoi(codecVideo->getCodecSpecifications()[DRing::Account::ConfProperties::CodecInfo::MAX_BITRATE])),
+            (unsigned)(jami::stoi(codecVideo->getCodecSpecifications()[DRing::Account::ConfProperties::CodecInfo::QUALITY])),
+            (unsigned)(jami::stoi(codecVideo->getCodecSpecifications()[DRing::Account::ConfProperties::CodecInfo::MIN_QUALITY])),
+            (unsigned)(jami::stoi(codecVideo->getCodecSpecifications()[DRing::Account::ConfProperties::CodecInfo::MAX_QUALITY])),
             videoBitrateInfo_.cptBitrateChecking,
             videoBitrateInfo_.maxBitrateChecking,
             videoBitrateInfo_.packetLostThreshold,
         };
     } else {
-        videoBitrateInfo_ = {0,0,0,0,0,0,0,0,0};
+        videoBitrateInfo_ = {0, 0, 0, 0, 0, 0, 0,
+                             MAX_ADAPTATIVE_BITRATE_ITERATION,
+                             PACKET_LOSS_THRESHOLD};
     }
 }
 
 void
 VideoRtpSession::storeVideoBitrateInfo() {
-    auto codecVideo = std::static_pointer_cast<ring::AccountVideoCodecInfo>(send_.codec);
+    auto codecVideo = std::static_pointer_cast<jami::AccountVideoCodecInfo>(send_.codec);
 
     if (codecVideo) {
         codecVideo->setCodecSpecifications({
-            {DRing::Account::ConfProperties::CodecInfo::BITRATE, ring::to_string(videoBitrateInfo_.videoBitrateCurrent)},
-            {DRing::Account::ConfProperties::CodecInfo::MIN_BITRATE, ring::to_string(videoBitrateInfo_.videoBitrateMin)},
-            {DRing::Account::ConfProperties::CodecInfo::MAX_BITRATE, ring::to_string(videoBitrateInfo_.videoBitrateMax)},
-            {DRing::Account::ConfProperties::CodecInfo::QUALITY, ring::to_string(videoBitrateInfo_.videoQualityCurrent)},
-            {DRing::Account::ConfProperties::CodecInfo::MIN_QUALITY, ring::to_string(videoBitrateInfo_.videoQualityMin)},
-            {DRing::Account::ConfProperties::CodecInfo::MAX_QUALITY, ring::to_string(videoBitrateInfo_.videoQualityMax)}
+            {DRing::Account::ConfProperties::CodecInfo::BITRATE, std::to_string(videoBitrateInfo_.videoBitrateCurrent)},
+            {DRing::Account::ConfProperties::CodecInfo::MIN_BITRATE, std::to_string(videoBitrateInfo_.videoBitrateMin)},
+            {DRing::Account::ConfProperties::CodecInfo::MAX_BITRATE, std::to_string(videoBitrateInfo_.videoBitrateMax)},
+            {DRing::Account::ConfProperties::CodecInfo::QUALITY, std::to_string(videoBitrateInfo_.videoQualityCurrent)},
+            {DRing::Account::ConfProperties::CodecInfo::MIN_QUALITY, std::to_string(videoBitrateInfo_.videoQualityMin)},
+            {DRing::Account::ConfProperties::CodecInfo::MAX_QUALITY, std::to_string(videoBitrateInfo_.videoQualityMax)}
         });
 
         if (histoQuality_.size() > MAX_SIZE_HISTO_QUALITY_)
@@ -535,36 +560,58 @@ VideoRtpSession::storeVideoBitrateInfo() {
 
     }
 }
-bool
-VideoRtpSession::setupRtcpChecker()
-{
-    getVideoBitrateInfo();
-    return true;
-}
-
-void
-VideoRtpSession::checkReceiver()
-{
-    if (receiveThread_ && receiveThread_->restartDecoder()) {
-        const auto& cid = callID_;
-        runOnMainThread([cid]{
-            if (auto call = Manager::instance().callFactory.getCall(cid)) {
-                call->restartMediaReceiver();
-            }
-        });
-    }
-}
 
 void
 VideoRtpSession::processRtcpChecker()
 {
-    checkReceiver();
     adaptQualityAndBitrate();
     rtcpCheckerThread_.wait_for(std::chrono::seconds(RTCP_CHECKING_INTERVAL));
 }
 
 void
-VideoRtpSession::cleanupRtcpChecker()
-{}
+VideoRtpSession::processPacketLoss()
+{
+    if (packetLossThread_.wait_for(RTCP_PACKET_LOSS_INTERVAL,
+                                   [this]{return socketPair_->rtcpPacketLossDetected();})) {
+        if (requestKeyFrameCallback_)
+            requestKeyFrameCallback_();
+    }
+}
 
-}} // namespace ring::video
+void
+VideoRtpSession::initRecorder(std::shared_ptr<MediaRecorder>& rec)
+{
+    if (receiveThread_) {
+        if (auto ob = rec->addStream(receiveThread_->getInfo())) {
+            receiveThread_->attach(ob);
+        }
+    }
+    if (auto input = std::static_pointer_cast<VideoInput>(videoLocal_)) {
+        if (auto ob = rec->addStream(input->getInfo())) {
+            input->attach(ob);
+        }
+    }
+}
+
+void
+VideoRtpSession::deinitRecorder(std::shared_ptr<MediaRecorder>& rec)
+{
+    if (receiveThread_) {
+        if (auto ob = rec->getStream(receiveThread_->getInfo().name)) {
+            receiveThread_->detach(ob);
+        }
+    }
+    if (auto input = std::static_pointer_cast<VideoInput>(videoLocal_)) {
+        if (auto ob = rec->getStream(input->getInfo().name)) {
+            input->detach(ob);
+        }
+    }
+}
+
+void
+VideoRtpSession::setChangeOrientationCallback(std::function<void(int)> cb)
+{
+    changeOrientationCallback_ = std::move(cb);
+}
+
+}} // namespace jami::video
